# Copyright (c) 2007 Thomas Lotze
# See also LICENSE.txt

"""zc.buildout recipe for building Postgres from source.
"""

import os.path

import gocept.cmmi

from cp.buildout_postgres import config


class Recipe(`gocept.cmmi.Recipe`):
    """zc.buildout recipe for building an Apache HTTP server from source.

    Configuration options:
	For gocept.cmmi:
		version
        url
        md5sum
        extra-options
        extra-vars
	for setup:
		


    Exported options:
        httpd-path
        envvars-path
        apxs-path
        module-dir
        htdocs
    """

    def __init__(self, buildout, name, options):
        super(Recipe, self).__init__(buildout, name, options)

        for key, value in config.items("postgres"):
            options.setdefault(key, value)

        location = options["location"]

        # Export some options.
        for name in "postgres", "envvars":
            options[name + "-path"] = os.path.join(location, "bin", name)
        options["module-dir"] = os.path.join(location, "modules")
        options["postgres-data"] = os.path.join(location, "data")

        options["extra-options"] = (" " +
                                    options.get("extra-options", ""))
	def install(self):
		options = self.options.copy()
		location = options["location"]
		return [location,]

	def update(self):
		pass


# Copyright (c) 2007 Thomas Lotze
# See also LICENSE.txt

"""zc.buildout recipe for building an Apache HTTP server from source.
"""

import os.path

import gocept.cmmi

from tl.buildout_postgres import config


class Recipe(gocept.cmmi.Recipe):
    """zc.buildout recipe for building an Apache HTTP server from source.

    Configuration options:
        url
        md5sum
        extra-options
        extra-vars

    Exported options:
        httpd-path
        envvars-path
        apxs-path
        module-dir
        htdocs
    """

    def __init__(self, buildout, name, options):
        super(Recipe, self).__init__(buildout, name, options)

        for key, value in config.items("httpd"):
            options.setdefault(key, value)

        location = options["location"]

        # Export some options.
        for name in "httpd", "envvars", "apxs":
            options[name + "-path"] = os.path.join(location, "bin", name)
        options["module-dir"] = os.path.join(location, "modules")
        options["htdocs"] = os.path.join(location, "htdocs")

        options["extra-options"] = ("--enable-so " +
                                    options.get("extra-options", ""))



#!python
"""Bootstrap setuptools installation

If you want to use setuptools in your package's setup.py, just include this
file in the same directory with it, and add this to the top of your setup.py::

    from ez_setup import use_setuptools
    use_setuptools()

If you want to require a specific version of setuptools, set a download
mirror, or use an alternate download directory, you can do so by supplying
the appropriate options to ``use_setuptools()``.

This file can also be run as a script to install or upgrade setuptools.
"""
import sys
DEFAULT_VERSION = "0.6c8"
DEFAULT_URL     = "http://pypi.python.org/packages/%s/s/setuptools/" % sys.version[:3]

md5_data = {
    'setuptools-0.6b1-py2.3.egg': '8822caf901250d848b996b7f25c6e6ca',
    'setuptools-0.6b1-py2.4.egg': 'b79a8a403e4502fbb85ee3f1941735cb',
    'setuptools-0.6b2-py2.3.egg': '5657759d8a6d8fc44070a9d07272d99b',
    'setuptools-0.6b2-py2.4.egg': '4996a8d169d2be661fa32a6e52e4f82a',
    'setuptools-0.6b3-py2.3.egg': 'bb31c0fc7399a63579975cad9f5a0618',
    'setuptools-0.6b3-py2.4.egg': '38a8c6b3d6ecd22247f179f7da669fac',
    'setuptools-0.6b4-py2.3.egg': '62045a24ed4e1ebc77fe039aa4e6f7e5',
    'setuptools-0.6b4-py2.4.egg': '4cb2a185d228dacffb2d17f103b3b1c4',
    'setuptools-0.6c1-py2.3.egg': 'b3f2b5539d65cb7f74ad79127f1a908c',
    'setuptools-0.6c1-py2.4.egg': 'b45adeda0667d2d2ffe14009364f2a4b',
    'setuptools-0.6c2-py2.3.egg': 'f0064bf6aa2b7d0f3ba0b43f20817c27',
    'setuptools-0.6c2-py2.4.egg': '616192eec35f47e8ea16cd6a122b7277',
    'setuptools-0.6c3-py2.3.egg': 'f181fa125dfe85a259c9cd6f1d7b78fa',
    'setuptools-0.6c3-py2.4.egg': 'e0ed74682c998bfb73bf803a50e7b71e',
    'setuptools-0.6c3-py2.5.egg': 'abef16fdd61955514841c7c6bd98965e',
    'setuptools-0.6c4-py2.3.egg': 'b0b9131acab32022bfac7f44c5d7971f',
    'setuptools-0.6c4-py2.4.egg': '2a1f9656d4fbf3c97bf946c0a124e6e2',
    'setuptools-0.6c4-py2.5.egg': '8f5a052e32cdb9c72bcf4b5526f28afc',
    'setuptools-0.6c5-py2.3.egg': 'ee9fd80965da04f2f3e6b3576e9d8167',
    'setuptools-0.6c5-py2.4.egg': 'afe2adf1c01701ee841761f5bcd8aa64',
    'setuptools-0.6c5-py2.5.egg': 'a8d3f61494ccaa8714dfed37bccd3d5d',
    'setuptools-0.6c6-py2.3.egg': '35686b78116a668847237b69d549ec20',
    'setuptools-0.6c6-py2.4.egg': '3c56af57be3225019260a644430065ab',
    'setuptools-0.6c6-py2.5.egg': 'b2f8a7520709a5b34f80946de5f02f53',
    'setuptools-0.6c7-py2.3.egg': '209fdf9adc3a615e5115b725658e13e2',
    'setuptools-0.6c7-py2.4.egg': '5a8f954807d46a0fb67cf1f26c55a82e',
    'setuptools-0.6c7-py2.5.egg': '45d2ad28f9750e7434111fde831e8372',
    'setuptools-0.6c8-py2.3.egg': '50759d29b349db8cfd807ba8303f1902',
    'setuptools-0.6c8-py2.4.egg': 'cba38d74f7d483c06e9daa6070cce6de',
    'setuptools-0.6c8-py2.5.egg': '1721747ee329dc150590a58b3e1ac95b',
}

import sys, os

def _validate_md5(egg_name, data):
    if egg_name in md5_data:
        from md5 import md5
        digest = md5(data).hexdigest()
        if digest != md5_data[egg_name]:
            print >>sys.stderr, (
                "md5 validation of %s failed!  (Possible download problem?)"
                % egg_name
            )
            sys.exit(2)
    return data


def use_setuptools(
    version=DEFAULT_VERSION, download_base=DEFAULT_URL, to_dir=os.curdir,
    download_delay=15
):
    """Automatically find/download setuptools and make it available on sys.path

    `version` should be a valid setuptools version number that is available
    as an egg for download under the `download_base` URL (which should end with
    a '/').  `to_dir` is the directory where setuptools will be downloaded, if
    it is not already available.  If `download_delay` is specified, it should
    be the number of seconds that will be paused before initiating a download,
    should one be required.  If an older version of setuptools is installed,
    this routine will print a message to ``sys.stderr`` and raise SystemExit in
    an attempt to abort the calling script.
    """
    was_imported = 'pkg_resources' in sys.modules or 'setuptools' in sys.modules
    def do_download():
        egg = download_setuptools(version, download_base, to_dir, download_delay)
        sys.path.insert(0, egg)
        import setuptools; setuptools.bootstrap_install_from = egg
    try:
        import pkg_resources
    except ImportError:
        return do_download()       
    try:
        pkg_resources.require("setuptools>="+version); return
    except pkg_resources.VersionConflict, e:
        if was_imported:
            print >>sys.stderr, (
            "The required version of setuptools (>=%s) is not available, and\n"
            "can't be installed while this script is running. Please install\n"
            " a more recent version first, using 'easy_install -U setuptools'."
            "\n\n(Currently using %r)"
            ) % (version, e.args[0])
            sys.exit(2)
        else:
            del pkg_resources, sys.modules['pkg_resources']    # reload ok
            return do_download()
    except pkg_resources.DistributionNotFound:
        return do_download()

def download_setuptools(
    version=DEFAULT_VERSION, download_base=DEFAULT_URL, to_dir=os.curdir,
    delay = 15
):
    """Download setuptools from a specified location and return its filename

    `version` should be a valid setuptools version number that is available
    as an egg for download under the `download_base` URL (which should end
    with a '/'). `to_dir` is the directory where the egg will be downloaded.
    `delay` is the number of seconds to pause before an actual download attempt.
    """
    import urllib2, shutil
    egg_name = "setuptools-%s-py%s.egg" % (version,sys.version[:3])
    url = download_base + egg_name
    saveto = os.path.join(to_dir, egg_name)
    src = dst = None
    if not os.path.exists(saveto):  # Avoid repeated downloads
        try:
            from distutils import log
            if delay:
                log.warn("""
---------------------------------------------------------------------------
This script requires setuptools version %s to run (even to display
help).  I will attempt to download it for you (from
%s), but
you may need to enable firewall access for this script first.
I will start the download in %d seconds.

(Note: if this machine does not have network access, please obtain the file

   %s

and place it in this directory before rerunning this script.)
---------------------------------------------------------------------------""",
                    version, download_base, delay, url
                ); from time import sleep; sleep(delay)
            log.warn("Downloading %s", url)
            src = urllib2.urlopen(url)
            # Read/write all in one block, so we don't create a corrupt file
            # if the download is interrupted.
            data = _validate_md5(egg_name, src.read())
            dst = open(saveto,"wb"); dst.write(data)
        finally:
            if src: src.close()
            if dst: dst.close()
    return os.path.realpath(saveto)




































def main(argv, version=DEFAULT_VERSION):
    """Install or upgrade setuptools and EasyInstall"""
    try:
        import setuptools
    except ImportError:
        egg = None
        try:
            egg = download_setuptools(version, delay=0)
            sys.path.insert(0,egg)
            from setuptools.command.easy_install import main
            return main(list(argv)+[egg])   # we're done here
        finally:
            if egg and os.path.exists(egg):
                os.unlink(egg)
    else:
        if setuptools.__version__ == '0.0.1':
            print >>sys.stderr, (
            "You have an obsolete version of setuptools installed.  Please\n"
            "remove it from your system entirely before rerunning this script."
            )
            sys.exit(2)

    req = "setuptools>="+version
    import pkg_resources
    try:
        pkg_resources.require(req)
    except pkg_resources.VersionConflict:
        try:
            from setuptools.command.easy_install import main
        except ImportError:
            from easy_install import main
        main(list(argv)+[download_setuptools(delay=0)])
        sys.exit(0) # try to force an exit
    else:
        if argv:
            from setuptools.command.easy_install import main
            main(argv)
        else:
            print "Setuptools version",version,"or greater has been installed."
            print '(Run "ez_setup.py -U setuptools" to reinstall or upgrade.)'

def update_md5(filenames):
    """Update our built-in md5 registry"""

    import re
    from md5 import md5

    for name in filenames:
        base = os.path.basename(name)
        f = open(name,'rb')
        md5_data[base] = md5(f.read()).hexdigest()
        f.close()

    data = ["    %r: %r,\n" % it for it in md5_data.items()]
    data.sort()
    repl = "".join(data)

    import inspect
    srcfile = inspect.getsourcefile(sys.modules[__name__])
    f = open(srcfile, 'rb'); src = f.read(); f.close()

    match = re.search("\nmd5_data = {\n([^}]+)}", src)
    if not match:
        print >>sys.stderr, "Internal error!"
        sys.exit(2)

    src = src[:match.start(1)] + repl + src[match.end(1):]
    f = open(srcfile,'w')
    f.write(src)
    f.close()


if __name__=='__main__':
    if len(sys.argv)>2 and sys.argv[1]=='--md5update':
        update_md5(sys.argv[2:])
    else:
        main(sys.argv[1:])









# Copyright (c) 2007 Thomas Lotze
# See also LICENSE.txt

import pkg_resources
import ConfigParser


config_file = pkg_resources.resource_stream(__name__, "defaults.cfg")
config = ConfigParser.ConfigParser()
config.readfp(config_file)



# Copyright (c) 2007 Thomas Lotze
# See also LICENSE.txt

"""Test suite for cp.buildout_postgres.
"""

import unittest

import zope.testing.doctest

import zc.buildout.testing


flags = (zope.testing.doctest.NORMALIZE_WHITESPACE |
         zope.testing.doctest.ELLIPSIS
         )


def setUp(test):
    zc.buildout.testing.buildoutSetUp(test)
    zc.buildout.testing.install("gocept.download", test)
    zc.buildout.testing.install("gocept.cmmi", test)
    zc.buildout.testing.install_develop("cp.buildout_postgres", test)


def test_suite():
    return unittest.TestSuite((
        zope.testing.doctest.DocFileSuite(
        "root.txt",
        setUp=setUp,
        tearDown=zc.buildout.testing.buildoutTearDown,
        package="cp.buildout_postgres",
        optionflags=flags,
        ),
        ))


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



#!/usr/bin/env python
#
# Copyright (c) 2007 Thomas Lotze
# See also LICENSE.txt

"""zc.buildout recipes for setting up a Postgres Environment.
"""

import os.path
import glob

from setuptools import setup, find_packages


project_path = lambda *names: os.path.join(os.path.dirname(__file__), *names)


entry_points = {
    "zc.buildout": [
    "postgres = cp.buildout_postgres.postgres:Recipe",
    "root = cp.buildout_postgres.root:Recipe",
    ],
    }

longdesc = open(project_path("README.txt")).read()

data_files = [("", glob.glob(project_path("*.txt")))]

install_requires = [
    "setuptools",
    "zc.buildout",
    "gocept.cmmi",
	"cns.recipe.symlink"
    ]

extras_require = {
    "test": ["zope.testing"],
    }

classifiers = [
    "Development Status :: 3 - Alpha",
    "Environment :: Console",
    "Environment :: Plugins",
    "Framework :: Buildout",
    "Intended Audience :: Developers",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: Zope Public License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Software Development :: Build Tools",
    "Topic :: System :: Software Distribution",
    ]

setup(name="cp.buildout_postgres",
      version="0.1",
      description=__doc__.strip(),
      long_description=longdesc,
      keywords="buildout zc.buildout recipe postgres database server instance",
      classifiers=classifiers,
      author="Craig Sawyer",
      author_email="csawyer@yumaed.org",
      url="http://pypi.python.org/pypi/cp.buildout_postgres",
      license="ZPL 2.1",
      packages=find_packages(),
      namespace_packages=["cp"],
      entry_points=entry_points,
      test_suite="cp.buildout_postgres.tests.test_suite",
      install_requires=install_requires,
      extras_require=extras_require,
      include_package_data=True,
      data_files=data_files,
      zip_safe=True,
      )



# Copyright (c) 2007 Thomas Lotze
# See also LICENSE.txt

"""zc.buildout recipe for Postgres.
"""

import os
import os.path
import stat
import subprocess, commands
import re
import gocept.cmmi
import cns.recipe.symlink
import urllib

import pkg_resources
import logging, zc.buildout

from cp.buildout_postgres import config


MOD_LINE = re.compile("^\s*mod_(.*)\.c\s*$")


read_resource = lambda filename: pkg_resources.resource_string(__name__,
															   filename)


class Recipe(object):
	"""zc.buildout recipe for postgres.
   
 	Configuration options:
	For gocept.cmmi:
		version
        url
        md5sum
        extra-options
        extra-vars

	for setup:
		log-dir
		data-dir
		pg-hba-extra
		pg-ident-extra
		pg-conf-extra


    Exported options:
        pg-path
		data-dir
		log-dir
		pid-dir
		startupscript
	"""

	def __init__(self, buildout, name, options):
		
		self.buildout = buildout
		self.name = name
		self.options = options
		self.log = logging.getLogger(self.name)
		self.log.info('constructor options: %s' % self.options)
		self.cmmi = gocept.cmmi.Recipe(buildout,name,options)
		self.symlink = cns.recipe.symlink.Recipe(buildout,name,options)
		#self.log.info(self.cmmi)
		#self.log.info(self.cmmi.options)
		#print dir(self.cmmi)
		for key, value in config.items("postgres"):
			self.log.info('default config: %s: %s' % (key,value))
			options.setdefault(key, value)
		options["location"] = os.path.join(
			buildout["buildout"]["parts-directory"], name)
		options["bin-dir"] = os.path.join(options["location"],'bin')
		options['conf-dir'] = os.path.join(options["location"],options["data-dir"])
		conf_path = os.path.join(options["location"], options["data-dir"])
		options['ctl_path'] = os.path.join(self.buildout["buildout"]["bin-directory"],self.name)
		options['lib-dir'] = os.path.join(options['location'],'lib')
		options["conf-path"] = conf_path
		options["serverroot"] = options["location"]
		# directories
		for sub in "",options["data-dir"], "lock", options["log-dir"], "run":
			path = os.path.join(options["location"], sub)
			options[sub] = path
		return

	def install(self):
		"""A recipe install method is expected to return a string, or an iterable of strings containing paths to be removed if a part is uninstalled."""
		try:
			options = self.options.copy()
			md5sum = urllib.urlopen(options['md5link']).read()
			#MD5 (postgresql-8.3.1.tar.gz) = 91ae66ca7a6051abe49d8cea737a9acc
			a,b = md5sum.split('=')
			md5sum = b.strip()
			self.options['md5sum'] = md5sum
			cmmiRet = self.cmmi.install()
			self.log.info('cmmiRet: %s' % cmmiRet)
			self.log.info('options in install are: %s' % options)
			location = options["location"]
			self.options['symlink_base'] = os.path.join(location,'bin')
			self.options['symlink_target'] = self.buildout["buildout"]["bin-directory"]
			self.options['symlink'] = "\n".join(os.listdir(self.options['symlink_base']))
			self.log.info('symlink: %s to %s' % (self.options['symlink_base'],self.options['symlink_target']))
			self.log.info('these items: %s' % self.options['symlink'])
			symRet = self.symlink.install()
			self.log.info('symRet: %s', symRet)

			self.log.info('starting in on directory additions')
			# directories
			for sub in "lock",options["data-dir"], options["log-dir"], "run":
				path = os.path.join(location, sub)
				if not os.path.exists(path):
					os.mkdir(path)
				else:
					assert os.path.isdir(path)
				options[sub] = path

			self.log.info('starting in on file additions')
			#files
			conf_path = options['conf-path']
			ctl_path = options['ctl_path']

			cmd = os.path.join(self.options['symlink_target'],'initdb')
		
			#args = '-E UTF8 --locale en_US.UTF-8','-D %s' % (os.path.join(location,options["data-dir"])) 
			cmd += ' -E UTF8 --locale en_US.UTF-8 -D %s' % (conf_path)
			ret = commands.getoutput(cmd)
			self.log.info('returning: %s' % ret)
			#ret = self.run(cmd,'-E UTF8','--locale=en_US.UTF-8','-D %s' % (conf_path))
			#self.log.info('run returned: ' % ret)
			self.log.info('setting up config files.')
			
			self.log.info("dir: %s" % os.listdir(conf_path))

			open(os.path.join(conf_path,'pg_hba.conf'), "w").write(read_resource("pg_hba.conf.in") % options)
			open(os.path.join(conf_path,'pg_ident.conf'), "w").write(read_resource("pg_ident.conf.in") % options)
			open(os.path.join(conf_path,'postgresql.conf'), "w").write(read_resource("postgresql.conf.in") % options)
			self.log.info('.finished config files.')
			os.chmod(conf_path,0700)
		except:
			e = LogException()
			self.log.info(e)
		self.log.info("dir: %s" % os.listdir(conf_path))
		return [location,ctl_path,conf_path] + symRet

	def update(self):
		pass

	def run(self,*args):
		cmd = ' '.join(args)
		self.log.info('running: %s' % cmd) 
		ret = commands.getoutput(cmd)
		# sl=os.spawnl
		# ret=apply(sl,(os.P_WAIT,args[0])+args)
		self.log.info('returning: %s' % ret)
		return ret
		# stdout, ignored = subprocess.Popen([cmd,args],stdout=subprocess.PIPE).communicate()
		# return stdout

def LogException():
	"""
	I grab the traceback log it
	And return a happy formatted message back to you

	"""
	import traceback
	msg = traceback.format_exc()
	print msg
	return msg